@@ -5,31 +5,36 @@
 
 module Reactive.Banana.Frameworks (
     -- * Synopsis
-    -- | Build event networks using existing event-based frameworks
-    -- and run them.
-    
+    -- | Connect to the outside world by building 'EventNetwork's
+    -- and running them.
+
     -- * Simple use
     interpretAsHandler,
 
-    -- * Building event networks with input/output
+    -- * Overview
     -- $build
-    compile, Frameworks,
+
+    -- * Building event networks with input/output
+    -- ** Core functions
+    compile, MomentIO,
     module Control.Event.Handler,
     fromAddHandler, fromChanges, fromPoll,
-    reactimate, Future, reactimate', initial, changes, imposeChanges,
-    FrameworksMoment(..), execute, liftIOLater,
+    reactimate, Future, reactimate',
+    changes,
+    -- $changes
+    imposeChanges,
+    execute, liftIOLater,
     -- $liftIO
     module Control.Monad.IO.Class,
-    
+
+    -- ** Utility functions
+    -- | This section collects a few convience functions
+    -- built from the core functions.
+    interpretFrameworks, newEvent, mapEventIO, newBehavior,
+
     -- * Running event networks
     EventNetwork, actuate, pause,
-    
-    -- * Utilities
-    -- $utilities
-    newEvent,
-    
-    -- * Internal
-    interpretFrameworks, showNetwork,
+
     ) where
 
 import           Control.Event.Handler
@@ -38,7 +43,6 @@ import           Control.Monad.IO.Class
 import           Data.IORef
 import           Reactive.Banana.Combinators
 import qualified Reactive.Banana.Internal.Combinators as Prim
-import           Reactive.Banana.Internal.Phantom
 import           Reactive.Banana.Types
 
 
@@ -58,29 +62,29 @@ The module presented here allows you to
 
 * perform /output/ in reaction to events.
 
-In constrast, the functions from "Reactive.Banana.Combinators" allow you 
+In contrast, the functions from "Reactive.Banana.Combinators" allow you
 to express the output events in terms of the input events.
 This expression is called an /event graph/.
 
 An /event network/ is an event graph together with inputs and outputs.
 To build an event network,
 describe the inputs, outputs and event graph in the
-'Moment' monad 
+'MomentIO' monad
 and use the 'compile' function to obtain an event network from that.
 
 To /activate/ an event network, use the 'actuate' function.
-The network will register its input event handlers and start 
+The network will register its input event handlers and start
 producing output.
 
 A typical setup looks like this:
-   
+
 > main = do
 >   -- initialize your GUI framework
 >   window <- newWindow
 >   ...
 >
 >   -- describe the event network
->   let networkDescription :: forall t. Frameworks t => Moment t ()
+>   let networkDescription :: MomentIO ()
 >       networkDescription = do
 >           -- input: obtain  Event  from functions that register event handlers
 >           emouse    <- fromAddHandler $ registerMouseEvent window
@@ -91,12 +95,12 @@ A typical setup looks like this:
 >           bdie      <- fromPoll       $ randomRIO (1,6)
 >
 >           -- express event graph
+>           behavior1 <- accumB ...
 >           let
->               behavior1 = accumB ...
 >               ...
 >               event15 = union event13 event14
->   
->           -- output: animate some event occurences
+>
+>           -- output: animate some event occurrences
 >           reactimate $ fmap print event15
 >           reactimate $ fmap drawCircle eventCircle
 >
@@ -112,24 +116,24 @@ The library uses this to register event handlers with your event-based framework
 
 * Use 'reactimate' to animate /output/ events.
 
+* Use 'compile' to put everything together in an 'EventNetwork's
+and use 'actuate' to start handling events.
+
 -}
 
 {-----------------------------------------------------------------------------
     Combinators
 ------------------------------------------------------------------------------}
-singletonsE :: Prim.Event a -> Event t a
-singletonsE = E . Prim.mapE (:[])
-
 {- | Output.
 Execute the 'IO' action whenever the event occurs.
 
 
 Note: If two events occur very close to each other,
-there is no guarantee that the @reactimate@s for one 
+there is no guarantee that the @reactimate@s for one
 event will have finished before the ones for the next event start executing.
 This does /not/ affect the values of events and behaviors,
 it only means that the @reactimate@ for different events may interleave.
-Fortuantely, this is a very rare occurrence, and only happens if
+Fortunately, this is a very rare occurrence, and only happens if
 
 * you call an event handler from inside 'reactimate',
 
@@ -137,7 +141,7 @@ Fortuantely, this is a very rare occurrence, and only happens if
 
 In these cases, the @reactimate@s follow the control flow
 of your event-based framework.
-    
+
 Note: An event network essentially behaves like a single,
 huge callback function. The 'IO' action are not run in a separate thread.
 The callback function will throw an exception if one of your 'IO' actions
@@ -145,16 +149,16 @@ does so as well.
 Your event-based framework will have to handle this situation.
 
 -}
-reactimate :: Frameworks t => Event t (IO ()) -> Moment t ()
-reactimate = M . Prim.addReactimate . Prim.mapE (return . sequence_) . unE
+reactimate :: Event (IO ()) -> MomentIO ()
+reactimate = MIO . Prim.addReactimate . Prim.mapE return . unE
 
 -- | Output.
 -- Execute the 'IO' action whenever the event occurs.
 --
 -- This version of 'reactimate' can deal with values obtained
 -- from the 'changes' function.
-reactimate' :: Frameworks t => Event t (Future (IO ())) -> Moment t ()
-reactimate' = M . Prim.addReactimate . Prim.mapE (unF . fmap sequence_ . sequence) . unE
+reactimate' :: Event (Future (IO ())) -> MomentIO ()
+reactimate' = MIO . Prim.addReactimate . Prim.mapE unF . unE
 
 
 -- | Input,
@@ -163,8 +167,8 @@ reactimate' = M . Prim.addReactimate . Prim.mapE (unF . fmap sequence_ . sequenc
 -- When the event network is actuated,
 -- this will register a callback function such that
 -- an event will occur whenever the callback function is called.
-fromAddHandler :: Frameworks t => AddHandler a -> Moment t (Event t a)
-fromAddHandler = M . fmap singletonsE . Prim.fromAddHandler
+fromAddHandler ::AddHandler a -> MomentIO (Event a)
+fromAddHandler = MIO . fmap E . Prim.fromAddHandler
 
 -- | Input,
 -- obtain a 'Behavior' by frequently polling mutable data, like the current time.
@@ -174,78 +178,102 @@ fromAddHandler = M . fmap singletonsE . Prim.fromAddHandler
 --
 -- This function is occasionally useful, but
 -- the recommended way to obtain 'Behaviors' is by using 'fromChanges'.
--- 
+--
 -- Ideally, the argument IO action just polls a mutable variable,
 -- it should not perform expensive computations.
 -- Neither should its side effects affect the event network significantly.
-fromPoll :: Frameworks t => IO a -> Moment t (Behavior t a)
-fromPoll = M . fmap B . Prim.fromPoll
+fromPoll :: IO a -> MomentIO (Behavior a)
+fromPoll = MIO . fmap B . Prim.fromPoll
 
 -- | Input,
 -- obtain a 'Behavior' from an 'AddHandler' that notifies changes.
--- 
+--
 -- This is essentially just an application of the 'stepper' combinator.
-fromChanges :: Frameworks t => a -> AddHandler a -> Moment t (Behavior t a)
-fromChanges initial changes = stepper initial <$> fromAddHandler changes
+fromChanges :: a -> AddHandler a -> MomentIO (Behavior a)
+fromChanges initial changes = do
+    e <- fromAddHandler changes
+    stepper initial e
 
 -- | Output,
--- observe the initial value contained in a 'Behavior'.
-initial :: Behavior t a -> Moment t a
-initial = M . Prim.initialB . unB
-
--- | Output,
--- observe when a 'Behavior' changes.
--- 
--- Strictly speaking, a 'Behavior' denotes a value that
--- varies /continuously/ in time,
--- so there is no well-defined event which indicates when the behavior changes.
--- 
--- Still, for reasons of efficiency, the library provides a way to observe
--- changes when the behavior is a step function, for instance as 
--- created by 'stepper'. There are no formal guarantees,
--- but the idea is that
+-- return an 'Event' that is adapted to the changes of a 'Behavior'.
 --
--- > changes (stepper x e) = return (calm e)
+-- Remember that semantically, a 'Behavior' is a function @Behavior a = Time -> a@.
+-- This means that a Behavior does not have a notion of \"changes\" associated with it.
+-- For instance, the following Behaviors are equal:
 --
--- Note: The values of the event will not become available
--- until event processing is complete.
+-- > stepper 0 []
+-- > = stepper 0 [(time1, 0), (time2, 0)]
+-- > = stepper 0 $ zip [time1,time2..] (repeat 0)
+--
+-- In principle, to perform IO actions with the value of a Behavior,
+-- one has to sample it using an 'Event' and the 'apply' function.
+--
+-- However, in practice, Behaviors are usually step functions.
+-- For reasons of efficiency, the library provides a way
+-- to obtain an Event that /mostly/ coincides with the steps of a Behavior,
+-- so that sampling is only done at a few select points in time.
+-- The idea is that
+--
+-- > changes =<< stepper x e  =  return e
+--
+-- Please use 'changes' only in a ways that do /not/ distinguish
+-- between the different expressions for the same Behavior above.
+--
+-- Note that the value of the event is actually the /new/ value,
+-- i.e. that value slightly after this point in time. (See the documentation of 'stepper').
+-- This is more convenient.
+-- However, the value will not become available until after event processing is complete;
+-- this is indicated by the type 'Future'.
 -- It can be used only in the context of 'reactimate''.
-changes :: Frameworks t => Behavior t a -> Moment t (Event t (Future a))
-changes = return . fmap F . singletonsE . Prim.changesB . unB
+changes :: Behavior a -> MomentIO (Event (Future a))
+changes = return . E . Prim.mapE F . Prim.changesB . unB
+
+{- $changes
+
+Note: If you need a variant of the 'changes' function that does /not/
+have the additional 'Future' type, then the following code snippet
+may be useful:
+
+> plainChanges :: Behavior a -> MomentIO (Event a)
+> plainChanges b = do
+>     (e, handle) <- newEvent
+>     eb <- changes b
+>     reactimate' $ (fmap handle) <$> eb
+>     return e
+
+However, this approach is not recommended, because the result 'Event'
+will occur /slightly/ later than the event returned by 'changes'.
+In fact, there is no guarantee whatsoever about what /slightly/ means
+in this context. Still, it is useful in some cases.
+
+-}
 
 -- | Impose a different sampling event on a 'Behavior'.
 --
--- The 'Behavior' will vary continuously as before, but the event returned
+-- The 'Behavior' will have the same values as before, but the event returned
 -- by the 'changes' function will now happen simultaneously with the
 -- imposed event.
 --
 -- Note: This function is useful only in very specific circumstances.
-imposeChanges :: Frameworks t => Behavior t a -> Event t () -> Behavior t a
+imposeChanges :: Behavior a -> Event () -> Behavior a
 imposeChanges b e = B $ Prim.imposeChanges (unB b) (Prim.mapE (const ()) (unE e))
 
--- | Dummy type needed to simulate impredicative polymorphism.
-newtype FrameworksMoment a
-    = FrameworksMoment
-    { runFrameworksMoment :: forall t. Frameworks t => Moment t a }
-
-unFM :: FrameworksMoment a -> Moment (FrameworksD,t) a
-unFM = runFrameworksMoment
-
 -- | Dynamically add input and output to an existing event network.
 --
--- Note: You can even do 'IO' actions here, but there is no
--- guarantee about the order in which they are executed.
-execute
-    :: Frameworks t
-    => Event t (FrameworksMoment a)
-    -> Moment t (Event t a)
-execute = M
-    . fmap singletonsE . Prim.executeE
-    . Prim.mapE (fmap last . sequence . map (unM . unFM) )
-    . unE
+-- Note: You can even do 'IO' actions here,
+-- which is useful if you want to register additional event handlers
+-- dynamically.
+-- However, there is no
+-- guarantee about the order in which the actions are executed.
+-- If the result 'Event' of this function is garbage collected,
+-- it may also happen that the actions are not executed at all.
+-- If you want a reliable way to turn events into 'IO' actions
+-- use the 'reactimate' and 'reactimate'' functions.
+execute :: Event (MomentIO a) -> MomentIO (Event a)
+execute = MIO . fmap E . Prim.executeE . Prim.mapE unMIO . unE
 
 -- $liftIO
--- 
+--
 -- > liftIO :: Frameworks t => IO a -> Moment t a
 --
 -- Lift an 'IO' action into the 'Moment' monad.
@@ -253,17 +281,14 @@ execute = M
 -- | Lift an 'IO' action into the 'Moment' monad,
 -- but defer its execution until compilation time.
 -- This can be useful for recursive definitions using 'MonadFix'.
-liftIOLater :: Frameworks t => IO () -> Moment t ()
-liftIOLater = M . Prim.liftIOLater
+liftIOLater :: IO () -> MomentIO ()
+liftIOLater = MIO . Prim.liftIOLater
 
 -- | Compile the description of an event network
 -- into an 'EventNetwork'
 -- that you can 'actuate', 'pause' and so on.
---
--- Event networks are described in the 'Moment' monad
--- and use the 'Frameworks' class constraint.
-compile :: (forall t. Frameworks t => Moment t ()) -> IO EventNetwork
-compile m = fmap EN $ Prim.compile $ unM (m :: Moment (FrameworksD, t) ())
+compile :: MomentIO () -> IO EventNetwork
+compile = fmap EN . Prim.compile . unMIO
 
 {-----------------------------------------------------------------------------
     Running event networks
@@ -293,70 +318,87 @@ actuate = Prim.actuate . unEN
 pause :: EventNetwork -> IO ()
 pause   = Prim.pause . unEN
 
--- | A multiline description of the current 'Latch'es and 'Pulse's in
--- the 'EventNetwork'.
---
--- Incidentally, evaluation the returned string to normal
--- form will also force the 'EventNetwork' to some kind of normal form.
--- This may be useful for benchmarking purposes.
-showNetwork :: EventNetwork -> IO String
-showNetwork = Prim.showNetwork . unEN
-
-{-----------------------------------------------------------------------------
-    Simple use
-------------------------------------------------------------------------------}
--- | Interpret by using a framework internally.
--- Only useful for testing library internals.
-interpretFrameworks :: (forall t. Event t a -> Event t b) -> [a] -> IO [[b]]
-interpretFrameworks f xs = do
-    output                    <- newIORef []
-    (addHandler, runHandlers) <- newAddHandler
-    network                   <- compile $ do
-        e <- fromAddHandler addHandler
-        reactimate $ fmap (\b -> modifyIORef output (++[b])) (f e)
-
-    actuate network
-    bs <- forM xs $ \x -> do
-        runHandlers x
-        bs <- readIORef output
-        writeIORef output []
-        return bs
-    return bs
-
--- | Simple way to write a single event handler with
--- functional reactive programming.
-interpretAsHandler
-    :: (forall t. Event t a -> Event t b)
-    -> AddHandler a -> AddHandler b
-interpretAsHandler f addHandlerA = AddHandler $ \handlerB -> do
-    network <- compile $ do
-        e <- fromAddHandler addHandlerA
-        reactimate $ handlerB <$> f e
-    actuate network
-    return (pause network)
-
-
 {-----------------------------------------------------------------------------
     Utilities
 ------------------------------------------------------------------------------}
-{-$utilities
-
-    This section collects a few convenience functions
-    for unusual use cases. For instance:
-    
-    * The event-based framework you want to hook into is poorly designed
-    
-    * You have to write your own event loop and roll a little event framework
-
--}
-
--- | Build an 'Event' together with an 'IO' action that can 
+-- | Build an 'Event' together with an 'IO' action that can
 -- fire occurrences of this event. Variant of 'newAddHandler'.
--- 
+--
 -- This function is mainly useful for passing callback functions
 -- inside a 'reactimate'.
-newEvent :: Frameworks t => Moment t (Event t a, Handler a)
+newEvent :: MomentIO (Event a, Handler a)
 newEvent = do
     (addHandler, fire) <- liftIO $ newAddHandler
     e <- fromAddHandler addHandler
     return (e,fire)
+
+-- | Build a 'Behavior' together with an 'IO' action that can
+-- update this behavior with new values.
+--
+-- Implementation:
+--
+-- > newBehavior a = do
+-- >     (e, fire) <- newEvent
+-- >     b         <- stepper a e
+-- >     return (b, fire)
+newBehavior :: a -> MomentIO (Behavior a, Handler a)
+newBehavior a = do
+    (e, fire) <- newEvent
+    b         <- stepper a e
+    return (b, fire)
+
+-- | Build a new 'Event' that contains the result
+-- of an IO computation.
+-- The input and result events will /not/ be simultaneous anymore,
+-- the latter will occur /later/ than the former.
+--
+-- Please use the 'fmap' for 'Event' if your computation is pure.
+--
+-- Implementation:
+--
+-- > mapEventIO f e1 = do
+-- >     (e2, handler) <- newEvent
+-- >     reactimate $ (\a -> f a >>= handler) <$> e1
+-- >     return e2
+mapEventIO :: (a -> IO b) -> Event a -> MomentIO (Event b)
+mapEventIO f e1 = do
+    (e2, handler) <- newEvent
+    reactimate $ (\a -> f a >>= handler) <$> e1
+    return e2
+
+{-----------------------------------------------------------------------------
+    Simple use
+------------------------------------------------------------------------------}
+-- | Interpret an event processing function by building an 'EventNetwork'
+-- and running it. Useful for testing, but uses 'MomentIO'.
+-- See 'interpret' for a plain variant.
+interpretFrameworks :: (Event a -> MomentIO (Event b)) -> [Maybe a] -> IO [Maybe b]
+interpretFrameworks f xs = do
+    output                    <- newIORef Nothing
+    (addHandler, runHandlers) <- newAddHandler
+    network                   <- compile $ do
+        e1 <- fromAddHandler addHandler
+        e2 <- f e1
+        reactimate $ writeIORef output . Just <$> e2
+
+    actuate network
+    bs <- forM xs $ \x -> do
+        case x of
+            Nothing -> return Nothing
+            Just x  -> do
+                runHandlers x
+                b <- readIORef output
+                writeIORef output Nothing
+                return b
+    return bs
+
+-- | Simple way to write a single event handler with
+-- functional reactive programming.
+interpretAsHandler :: (Event a -> Moment (Event b)) -> AddHandler a -> AddHandler b
+interpretAsHandler f addHandlerA = AddHandler $ \handlerB -> do
+    network <- compile $ do
+        e1 <- fromAddHandler addHandlerA
+        e2 <- liftMoment (f e1)
+        reactimate $ handlerB <$> e2
+    actuate network
+    return (pause network)
